/*  09.02.01 - main.nut
 *
 *  This file is part of Trans AI
 *
 *  Copyright 2009 fanio zilla <fanio.zilla@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */

/**
 * Trans AI global storage
 */
TransAI <- {
	/** Root Table */
	Root = null,
	/** Memory */
	Memory = null,
	/** Station Manager */
	StationMan = null,
	/** Vehicle Manager */
	VehicleMan = null,
	/** Service Manager */
	ServiceMan = null,
	/** Serv-able Manager */
	ServableMan = null,
	/** Company Manager */
	CompanyMan = null,
	/** Task Manager */
	TaskMan = null,
	/** Our Bank Balance */
	Balance = null,
	/** Accountant */
	Accountant = null,
	/**  Vehicle preview */
	NewVehicle = {},
};

require("dependencies.nut");

/**
 * extending AIController class
 */
class Trans extends AIController
{
	/** Event Checker */
	EventCheck = null;

    constructor() {
		TransAI.Root = this;
		TransAI.Memory = Memory();
        TransAI.StationMan = StationManager();
        TransAI.VehicleMan = VehicleManager();
		TransAI.ServiceMan = ServiceManager();
		TransAI.ServableMan = ServableManager();
	    TransAI.CompanyMan = CompanyManager();
		TransAI.TaskMan = TaskManager();
		TransAI.Accountant = AIAccounting();
		EventCheck = Task.Events();
    }



	/**
	 * Start main AI class
	 */
	function Start()
	{
	    Sleep(5);
	    TransAI.CompanyMan.Born();
	    TransAI.CompanyMan.Test();
		local wait_time = 0;
	    local sleep_time = 10;
		AILog.Info("Init task schedule");
		TransAI.TaskMan.New(EventCheck, 5);
		TransAI.TaskMan.New(Task.HeadQuarter(), 5);
		TransAI.TaskMan.New(Task.Monitor(), 5);
		TransAI.TaskMan.New(Task.PayLoan(), 10);
		TransAI.TaskMan.New(Task.GenerateDropOff(), 1);
		TransAI.TaskMan.New(Task.CurrentValue(), 20);
		TransAI.TaskMan.New(Task.GenerateService(), 5);
		TransAI.TaskMan.New(Task.Maintenance(), 10);
		TransAI.TaskMan.New(Task.Inflation(), 50);

	    /*
	    * ============ Main Loop ================
	    */

		try {
			do {
				wait_time += 10;
				/* run the task manager */
				TransAI.TaskMan.Run();
				if (wait_time % 100 == 0) TransAI.TaskMan.SetSleep(Assist.SleepTime());
				if (wait_time % 110 == 0) throw "died";
			} while (TransAI.Memory.Live);
		} catch (msg) {
			AILog.Warning("Error catched:" + msg);
		}
	    /*
	    * ====================================
	    * Out of loop mean something goes wrong. Destructor called
		*/
		Const = null;
		TransAI = null;
		AILog.Warning("Please make a bug report on:");
		AILog.Info("http://www.tt-forums.net/viewtopic.php?f=65&t=42272");
		AILog.Info("or");
	}

	/**
	 * Handle save game of OpenTTD
	 */
	function Save()
	{
		try {
			/* clearing events */
			this.EventCheck.Execute();
			local save_table = {};
			save_table.rawset("memory", TransAI.Memory.GetStorage());
			save_table.rawset(TransAI.ServiceMan.GetClassName(), TransAI.ServiceMan.GetStorage());
			AILog.Info("--- (partial) Save supported ---");
			return save_table;
		} catch (msg) {
			AILog.Warning("Can't save " + msg);
			return {};
		}
	}

	/**
	 * Handle loading savegame by OpenTTD
	 */
	function Load(version, data)
	{
	    AILog.Warning("::(experimental) Loading from version " + version);
		try TransAI.Memory.SetStorage(data.rawget("memory"))
		catch (x) AILog.Warning("Failed load: Memory "+ x);
		try TransAI.ServiceMan.SetStorage(data.rawget(TransAI.ServiceMan.GetClassName()))
		catch (x) AILog.Warning("Failed load: Service Manager "+ x);
	}
}
