/*  09.02.06 - builder.nut
 *  This file is part of Trans AI
 *
 *  Copyright 2009 fanio zilla <fanio.zilla@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */

/**
 * Class Building
 * super class of all builder
 */
class Building {

    _lastCost = null;               /// store the last cost of command
    _costHandler = null;            /// AIAccounting instance
    _test_flag = null;              /// flag indicator test mode
    _commander = null;            /// My Bos instance
    State = null;                 /// The state of this builder
    Road = null;                   /// RoadBuilder Class
    Rail = null;                 /// RailBuilder class
    ff_factor = null;              /// Cost ff_factor

    constructor(commander) {
        _lastCost = 0;
        _costHandler = AIAccounting();
        _test_flag = false;
        _commander = commander;
        State = Building.state(this);
        Road = Building.road(this);
        Rail = Building.rail(this);
        ff_factor = _commander.ff_factor;
        AIRail.SetCurrentRailType(AIRailTypeList().Begin());
    }
}


/**
 * Grouping vehicle
 * @param serv Service tabel
 */
 function Building::MakeGroup(serv)
 {
    local grp = AIGroup.CreateGroup(serv.VehicleType);
    local g_name = this._commander.randomizer +":" + serv.ID;
    if (!AIGroup.SetName(grp, g_name)) {
        local grp_list = AIGroupList();
        grp_list.Valuate(AIGroup.GetVehicleType);
        grp_list.KeepValue(serv.VehicleType);
        foreach (idx, val in grp_list) {
            if (AIGroup.GetName(idx) == g_name) {
                AIGroup.DeleteGroup(grp);
                grp = idx;
                break;
            }
        }
    }
    AIGroup.MoveVehicle(grp, serv.MainVhcID);
 }

/**
 * Servicing a route defined before by Road Vehicle
 */
function Building::RoadServicing(serv)
{
    AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_ROAD);
    //AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_TRAM);
    serv.Source.Refresh();
    serv.Destination.Refresh();
    serv.SourceDepot <- Platform();
    serv.DestinationDepot <- Platform();
    serv.SourceStation <- Stations();
    serv.DestinationStation <- Stations();
    local service_cost = 0;

    AILog.Warning("Test Mode");
    this.Road.SetService(serv);
    this.State.TestMode = true;
    if (!this.Road.Vehicle()) return false;
    service_cost += this.State.LastCost * 2;
    if (!Bank.Get(service_cost)) return false;
    //this.State.TestMode = false;

    if (!this.Road.Path(0 false)) if (!this.Road.Path(0, true)) return false;
    if (!this.Road.Track(0)) return false;
    service_cost += this.State.LastCost * 1.5;
    if (!Bank.Get(service_cost)) return false;

    if (!this.Road.Station(true)) return false;
    service_cost += this.State.LastCost;
    if (!Bank.Get(service_cost)) return false;

    if (!this.Road.Station(false)) return false;
    service_cost += this.State.LastCost;
    if (!Bank.Get(service_cost)) return false;

    if (!this.Road.Depot(true)) return false;
    service_cost += this.State.LastCost;
    if (!Bank.Get(service_cost)) return false;

    if (!this.Road.Depot(false)) return false;
    service_cost += this.State.LastCost;

    /* don't continue if I've not enough money */
    if (!Bank.Get(service_cost)) return false;

    AILog.Warning("Real Mode");
    this.State.TestMode = false;

    if (!this.Road.Station(true)) return false;
    if (!this.Road.Station(false)) return false;

    if (!this.Road.Depot(true)) return false;
    if (!this.Road.Depot(false)) return false;

    if (!this.Road.Path(1, false)) {
        this.Road.Path(1, true);
        if (!this.Road.Track(1)) {
        this.Road.Path(1, true);
        if (!this.Road.Track(1)) return false;
        }
    }

    if (!this.Road.Path(2, false)) {
        this.Road.Path(2, true);
        if (!this.Road.Track(2)) {
        this.Road.Path(2, true);
        if (!this.Road.Track(2)) return false;
        }
    }

    if (!this.Road.Path(3, false)) {
        this.Road.Path(3, true);
        if (!this.Road.Track(3)) {
        this.Road.Path(3, true);
        if (!this.Road.Track(3)) return false;
        }
    }

    // I'm sad to return false after going so far
    if (!this.Road.Vehicle()) return false;

    AIVehicle.StartStopVehicle(this.Road.GetService().MainVhcID);
    this.MakeGroup(this.Road.GetService());
    if (!this.Road.GetService().IsSubsidy) this.RenameStation(this.Road.GetService());

    this.Road.Vehicle();
    /*update table of service*/
    this._commander.service_tables[this.Road.GetService().ID] <- this.Road.GetService();
    this._commander.serviced_route[this.Road.GetService().ID] <- true;
    return true;
}

/**
 * Servicing a route defined before by Rail Vehicle
 */
function Building::RailServicing(serv)
{
    AIRail.SetCurrentRailType(AIRailTypeList().Begin());
    serv.Source.Refresh();
    serv.Destination.Refresh();
    serv.SourceDepot <- Platform();
    serv.DestinationDepot <- Platform();
    serv.SourceStation <- Stations();
    serv.DestinationStation <- Stations();

    local service_cost = 0;

    AILog.Warning("Test Mode");
    this.Rail.SetService(serv);
    this.State.TestMode = true;

    if (!this.Rail.Vehicle()) return false;
    service_cost += this.State.LastCost * 2;
    if (!Bank.Get(service_cost)) return false;
    //this.State.TestMode = false;

    if (!this.Rail.Station(true)) return false;
    service_cost += this.State.LastCost * 1.5;
    if (!Bank.Get(service_cost)) return false;

    if (!this.Rail.Station(false)) return false;
    service_cost += this.State.LastCost * 1.5;
    if (!Bank.Get(service_cost)) return false;

    this.Rail.Path(0, true);
    this.Rail.Track(0);
    service_cost += this.State.LastCost;
    if (!Bank.Get(service_cost)) return false;

    if (!this.Rail.Depot(true)) return false;
    service_cost += this.State.LastCost * 2;


    /* don't continue if I've not enough money */
    if (!Bank.Get(service_cost)) return false;

    AILog.Warning("Real Mode");
    this.State.TestMode = false;

    if (!this.Rail.Station(true)) return false;
    if (!this.Rail.Station(false)) return false;

    this.Rail.Path(1, true);
    if (!this.Rail.Track(1)) return false;

    if (!this.Rail.Depot(true)) return false;
    if (!this.Rail.Depot(false)) return false;

    // I'm sad to return false after going so far
    if (!this.Rail.Vehicle()) return false;
    AIVehicle.StartStopVehicle(this.Rail.GetService().MainVhcID);

    this.MakeGroup();
    if (this.Rail.GetService().IsSubsidy) this.RenameStation();

    //if (!Assist.Connect_BackBone(this, ))
    this._commander.rail_backbones.push(this.Rail.GetService().ID);

    /*update table of service*/
    this._commander.service_tables[this.Rail.GetService().ID] <- this.Rail.GetService();
    this._commander.serviced_route[this.Rail.GetService().ID] <- true;
    return true;
}

/**
 *
 * The State of current builder
 */
class Building.state {
    _main = null;
    constructor(main)
    {
      this._main = main;
    }

    function _set(idx, val)
    {
        switch (idx) {
            case "LastCost" :
                this._main._lastCost = val;
                break;
            case "TestMode" :
                this._main._test_flag = val;
                break;
          default : throw("the index '" + idx + "' does not exist");
        }
    }

    function _get(idx)
    {
        switch (idx) {
            case "LastCost" : return this._main._lastCost;
            case "TestMode" : return this._main._test_flag;
            default : throw("the index '" + idx + "' does not exist");
        }
    }
}

/**
 * HeadQuarter builder.
 * Build my HQ on random suitable site if I haven't yet or
 * @return tile location
 */
class Task.HeadQuarter extends YieldTask
{
	constructor()
	{
		::YieldTask.constructor("Build HeadQuarter");
		this._repeat = false;
		this._removable = true;
	}

	function _exec()
	{
		local hq = AICompany.GetCompanyHQ(AICompany.COMPANY_SELF);
		if (AIMap.IsValidTile(hq)) return hq;

		local loc = AITownList();
		local tiles = null;
		loc.Valuate(AITown.GetPopulation);
		loc.KeepBottom(TransAI.Memory.ID);
		for (local town = loc.Begin(); loc.HasNext(); town = loc.Next()) {
			tiles = Tiles.Flat(Tiles.OfTown(town, Tiles.Radius(AITown.GetLocation(town), 10)));
			tiles.Valuate(AITile.IsBuildableRectangle, 2, 2);
			tiles.RemoveValue(0);
			foreach (location, val in tiles) {
				if (AICompany.BuildCompanyHQ (location)) return location;
				AISign.RemoveSign(AISign.BuildSign(location,"here"));
				yield location;
			}
		}
	}
}
