/*  09.02.08 - company.nut
 *
 *  This file is part of Trans AI
 *
 *  Copyright 2009 fanio zilla <fanio.zilla@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */

 /**
 * CompanyManager
 * Company management class
 */
class CompanyManager
{
	///< Flag to indicate if we need to rescan drop_point
	drop_point_validated = null;

	constructor()
	{
		this.drop_point_validated = false;

		TransAI.Memory.ID = null;	///< Used ID as randomizer
		TransAI.Memory.Live = null;	///< Should I keep alive or retire ?
		TransAI.Memory.Start_date = 0;	///< Company launching date
		TransAI.Memory.Name = "Fanioz";	///<  President name of my company
		TransAI.Memory.Factor = 0;	///< factor of fluctuation
		TransAI.Memory.Drop_off_point = {}; ///< Drop off table
		TransAI.Memory.Industry_Close = []; ///< Store will closing industry
		TransAI.Memory.New_Engines = []; ///< Store new available engines
		TransAI.Memory.Lost_Vehicle = []; ///< Store Lost_Vehicle marked
		TransAI.Memory.InTrouble = false; ///< Reminder if we are in trouble
	}

	/**
	 * Company Start-Up
	 */
	function Born()
	{
		/* Wake up .. */
		AICompany.SetAutoRenewStatus(true);
		AICompany.SetAutoRenewMonths(-12);
		AICompany.SetAutoRenewMoney(10000);
		AIGroup.EnableWagonRemoval(true);

		/* Detect saved session */
		if (TransAI.Memory.Start_date == 0) TransAI.Memory.Start_date = AIDate.GetCurrentDate();
		if (TransAI.Memory.Live == null) TransAI.Memory.Live = 1;
		if (TransAI.Memory.ID == null) TransAI.Memory.ID = AICompany.ResolveCompanyID(AICompany.COMPANY_SELF);
		if (AICompany.GetPresidentName(TransAI.Memory.ID) != TransAI.Memory.Name) {
			/* Set my name */
			local i = 1;
			if (!AICompany.SetPresidentName(TransAI.Memory.Name)) {
				while (!AICompany.SetPresidentName(TransAI.Memory.Name + " " + i + " (jr.)")) {
					i++;
					AIController.Sleep(1);
				}
			}
			TransAI.Memory.Name = AICompany.GetPresidentName(TransAI.Memory.ID);
			AICompany.SetName(TransAI.Memory.Name + " Trans Corp.");
		}
		/* greeting you */
		AILog.Info("" + AICompany.GetName(TransAI.Memory.ID) + " has been started since " + Assist.DateStr(TransAI.Memory.Start_date));
		AILog.Info("Powered by " + _version_);
		Settings.Version();
		Debug.ResultOf("Random factor", TransAI.Memory.ID);
		if (TransAI.Memory.Factor == 0) TransAI.Memory.Factor = AICompany.GetMaxLoanAmount() / 10000;
	}



	/**
	 * Place to test something(
	 * @return up to you
	 */
	function Test()
	{
		//AILog.Info("Testing...");
		//AILog.Info("done!");
	}
}

/**
 * Try to do transport service
 */
class Task.Service extends TaskItem
{
	constructor()
    {
        ::TaskItem.constructor("Servicing");
        this._removable = false;
    }
    function Service()
    {
        AILog.Info("Try servicing...");
		local serv_list = TransAI.ServiceMan.GetList();
		local serv = null, src = null, dst = null;
        for (local idx = serv_list.Begin(); serv_list.HasNext(); idx = serv_list.Next()) {
            AIController.Sleep(1);
			serv = Services.MakeClass(TransAI.ServiceMan.Item(idx));
            if (serv.Mem.IsServed) continue;
            if (serv.Mem.Skipped) continue;
            src = Servable.MakeClass(serv.info.Source);
            dst = Servable.MakeClass(serv.info.Destination);
            AILog.Warning(src.GetName() + " to " + dst.GetName() + ":" + AICargo.GetCargoLabel(serv.info.Cargo));
            serv.info.IsServed = true;
            //AILog.Info("Value:" + TransAI.ServiceMan.GetValue(id));
            //updating manager
            TransAI.ServiceMan.ChangeItem(idx, serv.GetStorage());
            break;
        }
        AILog.Info("done!");
    }
}

/**
 * Clearing events routine before save
 * @note Make sure, never use do command
 */
class Task.Events extends TaskItem
{
    constructor()
    {
        ::TaskItem.constructor("Check Events");
        this._removable = false;
    }

	function Execute()
    {
        while (AIEventController.IsEventWaiting()) {
            AILog.Info("Clearing Events...");
            local e = AIEventController.GetNextEvent();
            local si = null;

            switch (e.GetEventType()) {
                case AIEvent.AI_ET_SUBSIDY_OFFER:
                    AILog.Info("New Subsidy offered" );
                break;

                case AIEvent.AI_ET_SUBSIDY_OFFER_EXPIRED:
                    local esoe = AIEventSubsidyOfferExpired.Convert(e);
                    si = esoe.GetSubsidyID();
                    AILog.Info("SubsidyID " + si + " offer expired" );
                    si = Services.CreateID(AISubsidy.GetSource(si), AISubsidy.GetDestination(si), AISubsidy.GetCargoType(si));
                    si = TransAI.ServiceMan.FindKey(si);
                    if (si) {
                        esoe = Services.MakeClass(TransAI.ServiceMan.Item(si));
                        if (!esoe.Mem.IsServed) TransAI.ServiceMan.RemoveItem(si);
                    }
                break ;

                case AIEvent.AI_ET_SUBSIDY_AWARDED:
                    local esa = AIEventSubsidyAwarded.Convert(e);
                    si = esa.GetSubsidyID();
                    AILog.Info("SubsidyID " + si + " awarded");
                    if (AICompany.IsMine(AISubsidy.GetAwardedTo(si))) break;
                    si = Services.CreateID(AISubsidy.GetSource(si), AISubsidy.GetDestination(si), AISubsidy.GetCargoType(si));
                    si = TransAI.ServiceMan.FindKey(si);
                    if (si) {
                        esa = Services.MakeClass(TransAI.ServiceMan.Item(si));
                        if (!esa.Mem.IsServed) TransAI.ServiceMan.RemoveItem(si);
                    }
                break;

                case AIEvent.AI_ET_SUBSIDY_EXPIRED:
                    local ese = AIEventSubsidyExpired.Convert(e);
                    si = ese.GetSubsidyID();
                    AILog.Info("SubsidyID " + si + " expired");
                break;

                case AIEvent.AI_ET_TEST:
                    AILog.Info("Undocumented event!" );
                break;

                case AIEvent.AI_ET_ENGINE_PREVIEW:
                    local me = AIEventEnginePreview.Convert(e);
                    AILog.Info("New Vehicle come : " + me.GetName());
                    TransAI.TaskMan.New(Task.AcceptPreview(me), 1);
                break;

                case AIEvent.AI_ET_COMPANY_NEW:
                    local me = AIEventCompanyNew.Convert(e);
                    si = me.GetCompanyID();
                    AILog.Warning("Welcome " + AICompany.GetName(si));
                break;

                case AIEvent.AI_ET_COMPANY_MERGER:
                    local me = AIEventCompanyMerger.Convert(e);
                    si = AICompany.GetName(me.GetOldCompanyID()) + "<->" + AICompany.GetName(me.GetNewCompanyID());
                    AILog.Info("And now come, the merger between " + si);
                break;

                case AIEvent.AI_ET_COMPANY_IN_TROUBLE:
                    local me = AIEventCompanyInTrouble.Convert(e);
                    if (AICompany.IsMine(me.GetCompanyID())) {
                        TransAI.Memory.InTrouble = true;
                        AILog.Info("Going to sleep");
                    }
                break;

                case AIEvent.AI_ET_COMPANY_BANKRUPT:
                    local me = AIEventCompanyBankrupt.Convert(e);
                    si = me.GetCompanyID();
                    if (AICompany.IsMine(si)) {
                        TransAI.Memory.Live = 0;
                        AILog.Info("Going to sleep");
                    } else AILog.Info("Good bye " + AICompany.GetName(si));
                break;

                /*
                future version will redo pathfinding to avoid this
                */
                case AIEvent.AI_ET_VEHICLE_CRASHED:
                    /*
                    * local me = AIEventVehicleCrashed.Convert(e);
                    AILog.Info("");
                    me.
                    */
                break;

                case AIEvent.AI_ET_VEHICLE_LOST:
                    local me = AIEventVehicleLost.Convert(e);
                    local vhc_ID = me.GetVehicleID();
                    AILog.Info("Vehicle lost " + AIVehicle.GetName(vhc_ID));
                    TransAI.Memory.Lost_Vehicle.push(vhc_ID);
                break;

                case AIEvent.AI_ET_VEHICLE_WAITING_IN_DEPOT:
                    local me = AIEventVehicleWaitingInDepot.Convert(e);
                    /* sell the vehicle if it is old -- handled */
                    local vhc_ID = me.GetVehicleID();
                    AILog.Info(AIVehicle.GetName(vhc_ID) + " is waiting");
                    /* reserved for vehicle_refit feature in future */
                break;

                case AIEvent.AI_ET_VEHICLE_UNPROFITABLE:
                    local me = AIEventVehicleUnprofitable.Convert(e);
                    local vhc_ID = me.GetVehicleID();
                    AILog.Info(AIVehicle.GetName(vhc_ID) + " is unprofitable");
                break;

                case AIEvent.AI_ET_INDUSTRY_OPEN:
                    local me = AIEventIndustryOpen.Convert(e);
                    si = me.GetIndustryID();
                    AILog.Info("Congratulation on grand opening " + AIIndustry.GetName(si));
                break;

                case AIEvent.AI_ET_INDUSTRY_CLOSE:
                    local me = AIEventIndustryClose.Convert(e);
                    si = me.GetIndustryID();
                    AILog.Info("Sadly enough, Good bye "+ AIIndustry.GetName(si));
                    TransAI.Memory.Industry_Close.push(si);
                break;

                case AIEvent.AI_ET_ENGINE_AVAILABLE:
                    local me = AIEventEngineAvailable.Convert(e);
                    si = me.GetEngineID();
                    AILog.Info(AIEngine.GetName(si) + " Available");
                    TransAI.Memory.New_Engines.push(si);
                break;

                case AIEvent.AI_ET_STATION_FIRST_VEHICLE:
                    /*
                     * local me = .Convert(e);
                     * AILog.Info("");
                     * me.
                     */
                break;

                case AIEvent.AI_ET_DISASTER_ZEPPELINER_CRASHED:
                    /*
                     * local me = .Convert(e);
                     * AILog.Info("");
                     * me.
                     */
                break;

                case AIEvent.AI_ET_DISASTER_ZEPPELINER_CLEARED:
                    /*
                    * local me = .Convert(e);
                    * AILog.Info("");
                    * me.
                    */
                break;

                case AIEvent.AI_ET_INVALID:
                    AILog.Info("Dunno why it is there" );
                break;

                default : Debug.DontCallMe("Events");
            }
            e = null;
            si = null;
        }
        AILog.Info("No more events");
        return true;
    }
}

/**
 * Evaluate current company value
 */
class Task.CurrentValue extends TaskItem
{
	constructor()
	{
		::TaskItem.constructor("Current_Value");
		this._removable = false;
	}

	function Execute()
	{
		this._result = AICompany.GetCompanyValue(TransAI.Memory.ID);
		return (" at " + Assist.DateStr(AIDate.GetCurrentDate()) +" is " + this._result);
	}
}

/**
 * Update factor of inflation
 */
class Task.Inflation extends TaskItem
{
	constructor()
	{
		::TaskItem.constructor("Inflation");
		this._removable = false;
	}

	function Execute()
	{
		TransAI.Factor10 <- AICompany.GetMaxLoanAmount() / TransAI.Memory.Factor;
		return true;
	}
}

/**
 * Maintenace Infrastructure
 * @note possibly upgrade too
 */
class Task.Maintenance extends TaskItem
{
	constructor()
	{
		::TaskItem.constructor("Maintenance");
		this._removable = false;
	}

	function Execute()
	{
		AILog.Info("Maintenance...");
		AILog.Info("done!");
		return true;
	}
}

/**
 * Monitor changes
 */
class Task.Monitor extends TaskItem
{
    constructor()
    {
        ::TaskItem.constructor("Monitoring");
        this._removable = false;
    }

    function Execute()
    {
		return true;
    }
}

/**
 * Accept Preview
 */
class Task.AcceptPreview extends TaskItem
{
	/** Parent class of Preview event */
	par_class = null;

	/**
	 * class contructor
	 * @param pc Parent class of Preview event
	 */
    constructor(pc)
    {
        ::TaskItem.constructor("Accept_Preview");
        this._removable = true;
        par_class = pc;
    }

    function Execute()
    {
    	par_class.AcceptPreview();
    	AILog.Info("Preview Vehicle:" + par_class.GetName());
    	par_class = null;
    	return true;
    }
}
