/*  09.02.23 services.nut
 *
 *  This file is part of Trans AI
 *
 *  Copyright 2009 fanio zilla <fanio.zilla@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */

/**
 * Services container
 */
class Services
{
	/// Memory - the storage of class
	Mem = null;
	/// Ignored pathfindng AITileList();
	IgnoreTileList = null; 
	constructor()
	{
		Mem = Memory();
		/*
		/// < Key of service
		///< Source service
		///< Destination service
		///< Cargo service
		///< Rough distance service
		///< Actual distance service
		///< is Subsidy service
		/// < Path tabel storage
		///< Flag of served service
		///< Flag to indicate that we should skip this
		///< Last cost accounting money needed
		///< Priority of this service
		*/
		this.IgnoreTileList = AITileList();
		
	}

	static function New (src, dst, cargoid)
	{

		assert(src instanceof Servable);
		assert(dst instanceof Servable);
		local ptabel = {
			Depart = [],
			Arrive = [],
			PM = [],
			Test = [],
			Reserved = [],
		};
		local serv = Services();
		serv.Mem.ID = 0;
		serv.Mem.Path = [];
		for(local c = 0; c < 4; c++) serv.Mem.Path.push(ptabel);
		serv.Mem.Key = Services.CreateID(src.GetID(), dst.GetID(), cargoid);
		serv.Mem.Source = src.GetID();
		serv.Mem.Destination = dst.GetID();
		serv.Mem.Cargo = cargoid;
		serv.Mem.R_Distance = src.GetDistanceManhattanToTile(dst.GetLocation());
		serv.Mem.A_Distance = 0;
        serv.Mem.IsSubsidy = false;
        serv.Mem.IsServed = false;
        serv.Mem.Skipped = false;
        serv.Mem.PastCost = 0;
        serv.Mem.Priority = 0;
		
		return serv;
	}

    /**
     * Make service ID
     * @param source Source ID
     * @param dest Destination ID
     * @param cargo Cargo ID
     * @return ID of service table
     */
    static function CreateID(source, dest, cargo)
    {
        return Assist.LeadZero(TransAI.Memory.ID) + ":" +
            Assist.LeadZero(source) + ":" + Assist.LeadZero(dest) + ":" + Assist.LeadZero(cargo) ;
    }

	/**
	 * Make class from storage
	 * @param storage to build the class from
	 * @return Service class
	 */
	static function MakeClass(storage)
	{
		assert(typeof(storage) == "table");
		local tmp = Services();
		tmp.Mem.SetStorage(storage);
		return tmp;
	}
}

/**
 * class to analyze the cost needed
 */
class Services.Cost
{
	serv = null; //service
	infr = null;
	source = null;
	destination = null;
	constructor(s)
	{
		this.serv = s;
		this.source = Servable.MakeClass(this.serv.info.Source);
		this.destination = Servable.MakeClass(this.serv.info.Destination);
		this.infr = null;
	}

	function SetInfrastructure(infra)
	{
		this.infr = infra;
	}

	function Route()
	{
		local _from = [], _to = [];
		local bodies = Tiles.Buildable(this.source.GetArea());
		foreach (idx, val in bodies) {
			AIController.Sleep(1);
			if (!AIMap.IsValidTile(idx)) continue;
			local heads = Tiles.Buildable(Tiles.Flat(Tiles.Adjacent(idx)));
			foreach (head, val in heads) if (AIMap.IsValidTile(head)) _from.push([head, idx]);
		}
		bodies = Tiles.Buildable(this.destination.GetArea());
		foreach (idx, val in bodies) {
			AIController.Sleep(1);
			if (!AIMap.IsValidTile(idx)) continue;
			local heads = Tiles.Buildable(Tiles.Flat(Tiles.Adjacent(idx)));
			foreach (head, val in heads)  if (AIMap.IsValidTile(head)) _to.push([head, idx]);
		}
		serv.info.A_Distance = path.GetLength();
	}

	function Depot()
	{
	}

	function Station()
	{
	}

	function Vehicle()
	{
	}
}
