/*  09.03.08 - generator.nut
 *
 *  This file is part of Trans AI
 *
 *  Copyright 2009 fanio zilla <fanio.zilla@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */

/**
    *
    * Generator Class for Service, Subsidy and Position
    */
class Generate
{
    /**
     * Position Generator. Generate position for depot and station.
     * @param heads Area of tile list that become a head
     * @param center of area
     * @param centered center first or last
     * @return yield of position table (head, body, id)
     */
    static function Pos(heads, center, centered)
    {
        AIController.Sleep(1);
        heads.Valuate(AIMap.DistanceManhattan, center);
        heads.Sort(AIAbstractList.SORT_BY_VALUE, centered);
        for (local head = heads.Begin(); heads.HasNext(); head = heads.Next()) {
            AIController.Sleep(1);
            local bodies = Tiles.BodiesOf(head);
            foreach (body, val in bodies){
                AIController.Sleep(1);
                local _pos =  Platform();
                _pos.SetBody(body);
                _pos.SetHead(head);
                yield _pos;
            }
        }
        return false;
    }

    /**
     * Subsidy Generator. Generate table of subsidy service.
     */
    static function Subsidy(main)
    {
        /* add randomizer, not all trans AI instance would handle subsidy */
        local skip_this = main.GetID() % 3;
        if (skip_this == 0) return;
        local random_factor =  (1 + skip_this) * 30;
        local list = AISubsidyList();
        local subs_service = null;
        AILog.Info("Try to generate services from subsidies");
        list.Valuate(AISubsidy.IsValidSubsidy);
        list.RemoveValue(0);
        list.Valuate(AISubsidy.IsAwarded);
        list.RemoveValue(1);
        foreach (i, val in list) {
            AIController.Sleep(1);
            local d = AISubsidy.GetExpireDate(i) - AIDate.GetCurrentDate();
            if (d < random_factor) continue;
            local source = AISubsidy.GetSource(i);
            local dest = AISubsidy.GetDestination(i);
            subs_service = Services.NewTable(source, dest, AISubsidy.GetCargoType(i));
            /* avoid double service */
            if (subs_service.ID in main.serviced_route) continue;
            if (subs_service.ID in main.expired_route) continue;
            if (main.service_keys.Exists(subs_service.ID)) continue;
            /* remove when supported */
            if ((!AISubsidy.SourceIsTown(i) && AIIndustry.IsBuiltOnWater(source)) ||
                (!AISubsidy.DestinationIsTown(i) && AIIndustry.IsBuiltOnWater(dest))) continue;
            subs_service = Services.RefreshTable(subs_service);
            if (AIMap.DistanceMax(subs_service.Source.Location, subs_service.Destination.Location) < 10) continue;
            subs_service.IsSubsidy = true;
            AILog.Info("Subsidy scheduled");
            AILog.Info(subs_service.Readable);
            main.service_keys.Insert(subs_service.ID, d - random_factor);
            if (subs_service.ID in main.service_tables) continue;
            main.service_tables[subs_service.ID] <- subs_service;
            return;
        }
        AILog.Info("No more Subsidy");
        subs_service = null;
    }


}
/**
 * Yield Task to generate service
 */
class Task.GenerateService extends YieldTask
{
	constructor()
	{
		::YieldTask.constructor("Generating Service");
		this._repeat = false;
		this._removable = true;
	}

	function _exec()
    {
        AILog.Info("Generating Services ...");
        local min_priority = 10000000;
        local min_distance = 20;
        local min_production = 10;
        local mult_transported = 100;
        local serv = null;
        local last_type = null;
        local speed = 0;
        local estimated_time = 0;
		local src_lst = null;
		local src = null, dst = null;
        foreach(cargo, val in  Cargo.Sorted()) {
            AIController.Sleep(2);
			if (!(cargo in TransAI.Memory.Drop_off_point)) continue;
			//AILog.Info("Cargo type " + AICargo.GetCargoLabel(cargo));
			local target_is_town = Cargo.HasTownEffect(cargo);
			dst = Servable.New(TransAI.Memory.Drop_off_point[cargo], target_is_town);
			TransAI.ServableMan.New(dst);
			//AILog.Info("Target " + dst.GetName());
			local src_istown = Cargo.HasTownEffect(cargo) && Cargo.TownList_Producing(cargo).Count() > 0;
			if (src_istown) {
				/* working with town */
				src_lst = Cargo.TownList_Producing(cargo);
			} else {
				/* let's assume working with industry */
				src_lst = Assist.NotOnWater(AIIndustryList_CargoProducing(cargo));
			}
			src_lst.Valuate(Assist.ServiceCost, src_istown, cargo);
			src_lst.RemoveBelowValue(min_production);
            local c = 0;
			local key = "";
			local id = 0;
            foreach (source, val in src_lst) {
                AIController.Sleep(1);
				key = Services.CreateID(source, dst.GetID(), cargo);
				id = TransAI.ServiceMan.FindKey(key);
				if (id) {
					AILog.Info("Found duplicate");
					serv = Services.MakeClass(TransAI.ServiceMan.Item(id));
					if (serv.Mem.Skipped)  {
						serv.Mem.Skipped = false;
						TransAI.ServiceMan.ChangeItem(id, serv.Mem.GetStorage());
					}
				} else {
					src = Servable.New(source, src_istown);
					TransAI.ServableMan.New(src);
					serv = Services.New(src, dst, cargo);
					TransAI.ServiceMan.New(serv, AICargo.GetCargoIncome(cargo, 20, 200));
					//AILog.Info("Source:" + src.GetName());
					//speed = AIEngine.GetMaxSpeed(serv.Engines.Begin());
					//estimated_time = (serv.Distance * 429 / speed / 24) ;
					//serv.info.Priority = ;
					//AICargo.GetCargoIncome(cargo, distance, estimated_time));
					c++;
					AILog.Info(AICargo.GetCargoLabel(cargo) +
					   " from " + src.GetName()+ " to " + dst.GetName());
					yield c;
				}
				serv = null;
				src = null;
            }
			//AILog.Info("Source count:" + c);
        }
		TransAI.ServiceMan.SortValueDescending();
        AILog.Info("Service Generator Stopped");
    }
}

/**
 * Task to generate drop off point.
 * Generate table of drop off point by industry types.
 */
class Task.GenerateDropOff extends TaskItem
{
    constructor()
    {
        ::TaskItem.constructor("Generating DropOff");
        this._removable = true;
    }

    function Execute()
    {
        local tick = Ticker();
        AILog.Info("Generating Drop of points ... ");
        local random_factor = TransAI.Memory.ID;
        local destiny = AIList();
        local number = 0;
        foreach (cargo, val in AICargoList()) {
            AIController.Sleep(1);
            if (Cargo.HasTownEffect(cargo)) {
                destiny = AITownList();
                destiny.Valuate(Cargo.TownCanAccept, cargo);
                destiny.RemoveValue(0);
                destiny.Valuate(AITown.GetPopulation);
                destiny.Sort(AIAbstractList.SORT_BY_VALUE, true);
            } else {
                destiny = Assist.NotOnWater(AIIndustryList_CargoAccepting(cargo));
                destiny.Valuate(Assist.CargoCount);
                destiny.Sort(AIAbstractList.SORT_BY_VALUE, false);
            }
            if (destiny == null) continue;
            number = destiny.Count();
            if (number == 0) continue;
            local counter = 0;
            /* add a bit randomization support */
            for (local dst = destiny.Begin(); destiny.HasNext(); dst = destiny.Next()) {
                AIController.Sleep(1);
                if ((random_factor % number) == counter) {
                    TransAI.Memory.Drop_off_point[cargo] <- dst;
                    break;
                }
                counter++;
            }
            destiny = null;
        }
        /*
        foreach (idx, val in TransAI.Memory.Drop_off_point) {
          AILog.Info(AIIndustry.GetName(val) + ":" + AICargo.GetCargoLabel(idx));
        } */
        AILog.Info("Drop off point Finished in " + tick.Elapsed());
        return true;
    }
}
